@@ -22,14 +22,14 @@ notebook = function(dir = '.', ...) {
     dir, ..., site.dir = '.',
     build = build,
     response = function(req) {
-      path = sub('^/', '', req$PATH_INFO)
+      path = sub('^/', '', decode_path(req))
       if (!grepl('[.]Rnb$', path) || !file.exists(path)) return(serve_dir()(req))
       json  = readLines(path, encoding = 'UTF-8')
       tmpl  = readLines(system.file('resources', 'rnotebook.html', package = 'servr'))
       token = '%RNOTEBOOK_DATA%'
       if (length(i <- which(token == tmpl)) != 1) stop('Wrong notebook template')
       tmpl[i] = paste(json, collapse = '\n')
-      tmpl = paste(tmpl, collapse = '\r\n')
+      tmpl = paste2(tmpl)
       list(status = 200L, headers = list('Content-Type' = 'text/html'), body = tmpl)
     }
   )
