@@ -221,3 +221,12 @@ port_available = function(port, host = '127.0.0.1') {
   httpuv::stopServer(tmp)
   TRUE
 }
+
+# decode the requested path
+decode_path = function(req) httpuv::decodeURIComponent(req$PATH_INFO)
+
+paste2 = function(...) paste(c(...), collapse = '\r\n')
+
+file_size = function(path) file.info(path)[, 'size']
+
+read_raw = function(path) readBin(path, 'raw', file_size(path))
